use bevy::prelude::*;

#[derive(Resource, Debug, Default)]
pub struct AssetLoader {
    pub layouts: Layouts,
    pub images: Images,
    pub sounds: Sounds,
    pub fonts: Fonts,
}

#[derive(Resource, Debug, Default)]
pub struct Layouts {
    pub player: Handle<TextureAtlasLayout>,
    pub statue: Handle<TextureAtlasLayout>,
}

#[derive(Resource, Debug, Default)]
pub struct Images {
    pub player: Handle<Image>,
    pub statue: Handle<Image>,
}

#[derive(Resource, Debug, Default)]
pub struct Sounds {
    pub meow: Handle<AudioSource>,
}

#[derive(Resource, Debug, Default)]
pub struct Fonts {
    pub iosevkalytemin: Handle<Font>,
}

pub fn startup(
    mut assets: ResMut<AssetLoader>,
    asset_server: Res<AssetServer>,
    mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
) {
    let props = asset_server.load("img/Props.png");
    let player = asset_server.load("img/Player.png");

    let player_atlas = TextureAtlasLayout::from_grid(
        UVec2::new(32, 64),
        3,
        1,
        Some(UVec2 { x: 0, y: 0 }),
        Some(UVec2 { x: 0, y: 0 }),
    );
    let statue_atlas =
        TextureAtlasLayout::from_grid(UVec2::new(40, 74), 1, 1, None, Some(UVec2::new(443, 20)));

    let player_atlas_handle = texture_atlases.add(player_atlas);
    let statue_atlas_handle = texture_atlases.add(statue_atlas);

    *assets = AssetLoader {
        images: Images {
            player,
            statue: props.clone(),
        },
        layouts: Layouts {
            player: player_atlas_handle,
            statue: statue_atlas_handle,
        },

        sounds: Sounds {
            meow: asset_server.load("sfx/meow.wav"),
        },

        fonts: Fonts {
            iosevkalytemin: asset_server.load("font/iosevka-lyteterm-regular.subset.ttf"),
        },
    };
}
